/*
 * Copyright Ian Burnett 2005, 2006.
 *
 * This file is part of Ian's Interactive LCD controller (IILC).
 * 
 * IILC is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 *
 * IILC is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with IILC; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA.
 */

#if !defined(__DEVICE_H__)
#define __DEVICE_H__


typedef struct _tag_device * LPDEVICE;

typedef enum _tag_key_event {

    KEY_UP_PRESS      = 1,
    KEY_DOWN_PRESS    = 2,
    KEY_LEFT_PRESS    = 3,
    KEY_RIGHT_PRESS   = 4,
    KEY_ENTER_PRESS   = 5,
    KEY_EXIT_PRESS    = 6,
    KEY_UP_RELEASE    = 7,
    KEY_DOWN_RELEASE  = 8,
    KEY_LEFT_RELEASE  = 9,
    KEY_RIGHT_RELEASE = 10,
    KEY_ENTER_RELEASE = 11,
    KEY_EXIT_RELEASE  = 12

} PACKET_KEY_EVENT;



typedef void (* pfnFanReportListener)(void * context,
                                     unsigned int fan_id,
                                     unsigned int rpm);

typedef void (* pfnTempReportListener)(void * context,
                                      unsigned int sensor_id,
                                      double degrees_c);

typedef void (* pfnKeyEventListener)(void * context,
                                    PACKET_KEY_EVENT key_event);



LCD_API int deviceCreate(const char * pszClass,
                         const char * pszPort,
                         LPDEVICE * ppDevice);

LCD_API int deviceDispose(LPDEVICE * ppDevice);

LCD_API int deviceStart(LPDEVICE pDevice);

LCD_API int deviceStop(LPDEVICE pDevice);


LCD_API int deviceSetCallbackTemperature(LPDEVICE pDevice,
                                         pfnTempReportListener pfn, 
                                         void * context);

LCD_API int deviceSetCallbackFan(LPDEVICE pDevice, 
                                 pfnFanReportListener pfn, 
                                 void * context);

LCD_API int deviceSetCallbackKeyEvent(LPDEVICE pDevice, 
                                      pfnKeyEventListener pfn, 
                                      void * context);


LCD_API int deviceSetBacklight(LPDEVICE pdevice, 
                               unsigned char backlightValue);

LCD_API int deviceClear(LPDEVICE pdevice);

LCD_API int deviceSetContrast(LPDEVICE pdevice, 
                             unsigned char contrastValue);

LCD_API int deviceSetLED(LPDEVICE pDevice, 
                         unsigned int led_id, 
                         LPCOLOUR pColour);

LCD_API int deviceWriteText(LPDEVICE pdevice, 
                            int x, 
                            int y, 
                            char * strData, 
                            int width);

LCD_API int deviceSetTemperatureReporting(LPDEVICE pdevice, 
                                          unsigned int mask);

LCD_API int deviceSetFanReporting(LPDEVICE pdevice, 
                                  unsigned int mask);

LCD_API int deviceSetFanPower(LPDEVICE pdevice, 
                              unsigned char fan1, 
                              unsigned char fan2,
                              unsigned char fan3, 
                              unsigned char fan4);



#endif /* __DEVICE_H__ */

