%{
#include <stdio.h>

#include "config.h"

%}

%union
{
	int number;
	char * str;
}


%token OBRACE
%token EBRACE
%token QUOTE
%token SEMICOLON
%token DISPLAY    
%token DEVICE    
%token MODULE
%token LIBRARY
%token CONTROL
%token LED
%token TEMP
%token CLASS
%token PORT
%token REPORT
%token USE
%token FAN
%token AS 
%token SCREEN
%token AT
%token SHOW
%token SIZE_KW
%token COMMA
%token VIRTUAL

%token <str> STRING
%token <number> INTEGER


%error-verbose

%%

file:
	  /* empty */
	| file display
;


display:
	DISPLAY STRING 
		{ parserBeginDisplay($2); }
		OBRACE display_body EBRACE
		{ parserEndDisplay(); }
;

display_body:
	  /* empty */
	| display_body screen
	| display_body module
	| display_body device
;

device:
	DEVICE STRING
		{ parserDeviceBegin($2); }
		OBRACE device_body EBRACE
		{ parserDeviceEnd(); }
;

device_body:
	  /* empty */
	| device_body CLASS STRING SEMICOLON
		{ parserSetDeviceClass($3); }
	| device_body PORT STRING SEMICOLON
		{ parserSetDevicePort($3); }
	| device_body USE LED INTEGER AS STRING SEMICOLON
		{ parserDeviceUseLED($4, $6); }
	| device_body USE TEMP INTEGER AS STRING SEMICOLON
		{ parserDeviceUseTemp($4, $6); }
	| device_body USE FAN INTEGER AS STRING SEMICOLON
		{ parserDeviceUseFan($4, $6); }
;

module:
	MODULE STRING
		{ parserBeginModule($2); }
		OBRACE module_body EBRACE
		{ parserEndModule(); }
;

screen:
	SCREEN STRING
		{ parserBeginScreen($2); }
		OBRACE screen_body EBRACE
		{ parserEndScreen(); }
;

screen_body:
	  /* empty */

	  /* Module occupies all of the screen */
	| SHOW STRING SEMICOLON

	  /* Module is of specific size at (0,0) */
	| SHOW STRING SIZE_KW point SEMICOLON
	
	  /* Module is of specific size at specific location */
	| SHOW STRING SIZE_KW point AT point SEMICOLON
;

point:
	INTEGER COMMA INTEGER
;

module_body:
	  /* empty */
	| module_body LIBRARY STRING SEMICOLON
		{ parserSetModuleLibrary($3); }
	| module_body CONTROL STRING AS INTEGER SEMICOLON
		{ parserModuleControlObject($3, $5); }
	| module_body REPORT STRING AS INTEGER SEMICOLON
		{ parserModuleReportObject($3, $5); }
	| module_body VIRTUAL TEMP INTEGER AS STRING SEMICOLON
		{ parserModuleVirtualObject($4, $6); }
;

%%

int yyerror(char * s)
{
	fprintf (stdout, "%s\n", s);
	exit(1);
}

void startParse()
{
	yyparse();
}

