/*
 * Copyright Ian Burnett 2005, 2006.
 *
 * This file is part of Ian's Interactive LCD controller (IILC).
 * 
 * IILC is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 *
 * IILC is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with IILC; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA.
 */

#include "lcd.h"
#include "screen.h"
#include "module.h"
#include "string.h"

typedef struct _tag_window
{
    unsigned int x;

    unsigned int y;

    unsigned int width;

    unsigned int height;

    pfnGetTextData pfn;

    void * context;

} WINDOW, * LPWINDOW;


typedef enum
{
    SCREEN_TYPE_TEXT,
    SCREEN_TYPE_BAR

} SCREEN_TYPE;

typedef struct _tag_screen
{
    int type;

    int width;

    int height;

    char ** pScreenData;

    unsigned int ** nWindowMap;

    LPWINDOW * pWindowList;

} SCREEN;

int screenGetSize(LPSCREEN pScreen, int * width, int * height)
{
    *width = pScreen->width;
    *height = pScreen->height;

    return 0;
}


int screenCreate(LPSCREEN * ppScreen, int width, int height)
{
    int rc = 0, i = 0;
    LPSCREEN pScreen = NULL;

    pScreen = calloc(1, sizeof(SCREEN));

    /* Initialise all screen data to spaces */
    pScreen->pScreenData = malloc(sizeof(char *) * height);
    for (i = 0; i < height; i++) {
        pScreen->pScreenData[i] = malloc(sizeof(char) * width);
        memset(pScreen->pScreenData[i], (int) ' ', width);
    }
    
    /* Initialise the screen map to no controlling function (-1) */
    pScreen->nWindowMap = malloc(sizeof(unsigned int *) * height);
    for (i = 0; i < height; i++) {
        pScreen->nWindowMap[i] = malloc(sizeof(unsigned int) * width);
        memset(pScreen->nWindowMap[i], -1, width);
    }

    /* Assign to user */
    *ppScreen = pScreen;

    return rc;
}


int screenAddWindowText(LPSCREEN pScreen, int x, int y, int width, int height,
                        pfnGetTextData pfn, void * context)
{
    int rc = 0;





    return rc;
}



int screenDoRender(LPSCREEN pScreen, int y, char * pszScreen)
{
    int rc = 0;
    int i = 0;

    for (i = 0; i < pScreen->width; i++) {

        int nController = pScreen->nWindowMap[y][i];

        if (nController == -1) {
            /* No controlling function */
            pszScreen[i] = ' ';
        }
        else {
            LPWINDOW pWindow = NULL;
            char * buf = NULL;

            /* Get the window which controls this */
            pWindow = pScreen->pWindowList[nController];

            /* Perform the callback */
            (pWindow->pfn)(pWindow->context, pScreen, y, &buf);
            printf("Done callback: buf now %s\n", buf);
        }
    }

    return rc;
}


void * RenderThread(void * arg)
{
    int running = 0;

    printf("RenderThread\n");

    while (running)
    {


    }

    return NULL;
}


